# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.pagination import AsyncPager, SyncPager
from ...core.request_options import RequestOptions
from ...types.flow_execution_summary import FlowExecutionSummary
from ...types.get_flow_execution_request_parameters_hydrate_enum import GetFlowExecutionRequestParametersHydrateEnum
from ...types.get_flow_execution_response_content import GetFlowExecutionResponseContent
from ...types.list_flow_executions_paginated_response_content import ListFlowExecutionsPaginatedResponseContent
from .raw_client import AsyncRawExecutionsClient, RawExecutionsClient


class ExecutionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawExecutionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawExecutionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawExecutionsClient
        """
        return self._raw_client

    def list(
        self,
        flow_id: str,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[FlowExecutionSummary, ListFlowExecutionsPaginatedResponseContent]:
        """
        Parameters
        ----------
        flow_id : str
            Flow id

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[FlowExecutionSummary, ListFlowExecutionsPaginatedResponseContent]
            Flow executions successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.flows.executions.list(
            flow_id="flow_id",
            from_="from",
            take=1,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(flow_id, from_=from_, take=take, request_options=request_options)

    def get(
        self,
        flow_id: str,
        execution_id: str,
        *,
        hydrate: typing.Optional[
            typing.Union[
                GetFlowExecutionRequestParametersHydrateEnum,
                typing.Sequence[GetFlowExecutionRequestParametersHydrateEnum],
            ]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetFlowExecutionResponseContent:
        """
        Parameters
        ----------
        flow_id : str
            Flow id

        execution_id : str
            Flow execution id

        hydrate : typing.Optional[typing.Union[GetFlowExecutionRequestParametersHydrateEnum, typing.Sequence[GetFlowExecutionRequestParametersHydrateEnum]]]
            Hydration param

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetFlowExecutionResponseContent
            Flow execution successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.flows.executions.get(
            flow_id="flow_id",
            execution_id="execution_id",
            hydrate=["debug"],
        )
        """
        _response = self._raw_client.get(flow_id, execution_id, hydrate=hydrate, request_options=request_options)
        return _response.data

    def delete(
        self, flow_id: str, execution_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Parameters
        ----------
        flow_id : str
            Flows id

        execution_id : str
            Flow execution identifier

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.flows.executions.delete(
            flow_id="flow_id",
            execution_id="execution_id",
        )
        """
        _response = self._raw_client.delete(flow_id, execution_id, request_options=request_options)
        return _response.data


class AsyncExecutionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawExecutionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawExecutionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawExecutionsClient
        """
        return self._raw_client

    async def list(
        self,
        flow_id: str,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[FlowExecutionSummary, ListFlowExecutionsPaginatedResponseContent]:
        """
        Parameters
        ----------
        flow_id : str
            Flow id

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[FlowExecutionSummary, ListFlowExecutionsPaginatedResponseContent]
            Flow executions successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.flows.executions.list(
                flow_id="flow_id",
                from_="from",
                take=1,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(flow_id, from_=from_, take=take, request_options=request_options)

    async def get(
        self,
        flow_id: str,
        execution_id: str,
        *,
        hydrate: typing.Optional[
            typing.Union[
                GetFlowExecutionRequestParametersHydrateEnum,
                typing.Sequence[GetFlowExecutionRequestParametersHydrateEnum],
            ]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetFlowExecutionResponseContent:
        """
        Parameters
        ----------
        flow_id : str
            Flow id

        execution_id : str
            Flow execution id

        hydrate : typing.Optional[typing.Union[GetFlowExecutionRequestParametersHydrateEnum, typing.Sequence[GetFlowExecutionRequestParametersHydrateEnum]]]
            Hydration param

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetFlowExecutionResponseContent
            Flow execution successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.flows.executions.get(
                flow_id="flow_id",
                execution_id="execution_id",
                hydrate=["debug"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(flow_id, execution_id, hydrate=hydrate, request_options=request_options)
        return _response.data

    async def delete(
        self, flow_id: str, execution_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Parameters
        ----------
        flow_id : str
            Flows id

        execution_id : str
            Flow execution identifier

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.flows.executions.delete(
                flow_id="flow_id",
                execution_id="execution_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(flow_id, execution_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

ConnectionDisableSignup = bool
"""
When true, prevents new user registration through this connection. Existing users can still authenticate. Useful for invite-only applications or during user migration.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .connection_attribute_map_attributes import ConnectionAttributeMapAttributes
from .connection_attribute_map_userinfo_scope import ConnectionAttributeMapUserinfoScope
from .connection_mapping_mode_enum_oidc import ConnectionMappingModeEnumOidc


class ConnectionAttributeMapOidc(UniversalBaseModel):
    """
    Configuration for mapping claims from the identity provider to Auth0 user profile attributes. Allows customizing which IdP claims populate user fields and how they are transformed.
    """

    attributes: typing.Optional[ConnectionAttributeMapAttributes] = None
    mapping_mode: typing.Optional[ConnectionMappingModeEnumOidc] = None
    userinfo_scope: typing.Optional[ConnectionAttributeMapUserinfoScope] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

EventStreamCloudEventConnectionDeletedObject1OptionsAttributeMapAttributes = typing.Dict[str, typing.Any]
"""
Object containing mapping details for incoming claims
"""



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.get_signing_keys_response_content import GetSigningKeysResponseContent
from ...types.revoked_signing_keys_response_content import RevokedSigningKeysResponseContent
from ...types.rotate_signing_keys_response_content import RotateSigningKeysResponseContent
from ...types.signing_keys import SigningKeys
from .raw_client import AsyncRawSigningClient, RawSigningClient


class SigningClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSigningClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSigningClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSigningClient
        """
        return self._raw_client

    def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> typing.List[SigningKeys]:
        """
        Retrieve details of all the application signing keys associated with your tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[SigningKeys]
            The signing keys were retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.keys.signing.list()
        """
        _response = self._raw_client.list(request_options=request_options)
        return _response.data

    def rotate(self, *, request_options: typing.Optional[RequestOptions] = None) -> RotateSigningKeysResponseContent:
        """
        Rotate the application signing key of your tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RotateSigningKeysResponseContent
            Signing key rotated successfully.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.keys.signing.rotate()
        """
        _response = self._raw_client.rotate(request_options=request_options)
        return _response.data

    def get(
        self, kid: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetSigningKeysResponseContent:
        """
        Retrieve details of the application signing key with the given ID.

        Parameters
        ----------
        kid : str
            Key id of the key to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetSigningKeysResponseContent
            The signing keys were retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.keys.signing.get(
            kid="kid",
        )
        """
        _response = self._raw_client.get(kid, request_options=request_options)
        return _response.data

    def revoke(
        self, kid: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RevokedSigningKeysResponseContent:
        """
        Revoke the application signing key with the given ID.

        Parameters
        ----------
        kid : str
            Key id of the key to revoke

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RevokedSigningKeysResponseContent
            Signing key revoked successfully.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.keys.signing.revoke(
            kid="kid",
        )
        """
        _response = self._raw_client.revoke(kid, request_options=request_options)
        return _response.data


class AsyncSigningClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSigningClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSigningClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSigningClient
        """
        return self._raw_client

    async def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> typing.List[SigningKeys]:
        """
        Retrieve details of all the application signing keys associated with your tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[SigningKeys]
            The signing keys were retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.keys.signing.list()


        asyncio.run(main())
        """
        _response = await self._raw_client.list(request_options=request_options)
        return _response.data

    async def rotate(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RotateSigningKeysResponseContent:
        """
        Rotate the application signing key of your tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RotateSigningKeysResponseContent
            Signing key rotated successfully.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.keys.signing.rotate()


        asyncio.run(main())
        """
        _response = await self._raw_client.rotate(request_options=request_options)
        return _response.data

    async def get(
        self, kid: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetSigningKeysResponseContent:
        """
        Retrieve details of the application signing key with the given ID.

        Parameters
        ----------
        kid : str
            Key id of the key to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetSigningKeysResponseContent
            The signing keys were retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.keys.signing.get(
                kid="kid",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(kid, request_options=request_options)
        return _response.data

    async def revoke(
        self, kid: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RevokedSigningKeysResponseContent:
        """
        Revoke the application signing key with the given ID.

        Parameters
        ----------
        kid : str
            Key id of the key to revoke

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RevokedSigningKeysResponseContent
            Signing key revoked successfully.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.keys.signing.revoke(
                kid="kid",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.revoke(kid, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import encode_path_param
from ...core.pagination import AsyncPager, SyncPager
from ...core.parse_error import ParsingError
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...errors.bad_request_error import BadRequestError
from ...errors.forbidden_error import ForbiddenError
from ...errors.not_found_error import NotFoundError
from ...errors.too_many_requests_error import TooManyRequestsError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.list_user_sessions_paginated_response_content import ListUserSessionsPaginatedResponseContent
from ...types.session_response_content import SessionResponseContent
from pydantic import ValidationError


class RawSessionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        user_id: str,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[SessionResponseContent, ListUserSessionsPaginatedResponseContent]:
        """
        Retrieve details for a user's sessions.

        Parameters
        ----------
        user_id : str
            ID of the user to get sessions for

        from_ : typing.Optional[str]
            An optional cursor from which to start the selection (exclusive).

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[SessionResponseContent, ListUserSessionsPaginatedResponseContent]
            The sessions were retrieved
        """
        _response = self._client_wrapper.httpx_client.request(
            f"users/{encode_path_param(user_id)}/sessions",
            method="GET",
            params={
                "from": from_,
                "take": take,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListUserSessionsPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListUserSessionsPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.sessions
                _parsed_next = _parsed_response.next
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    user_id,
                    from_=_parsed_next,
                    take=take,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Delete all sessions for a user.

        Parameters
        ----------
        user_id : str
            ID of the user to get sessions for

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"users/{encode_path_param(user_id)}/sessions",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawSessionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        user_id: str,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[SessionResponseContent, ListUserSessionsPaginatedResponseContent]:
        """
        Retrieve details for a user's sessions.

        Parameters
        ----------
        user_id : str
            ID of the user to get sessions for

        from_ : typing.Optional[str]
            An optional cursor from which to start the selection (exclusive).

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[SessionResponseContent, ListUserSessionsPaginatedResponseContent]
            The sessions were retrieved
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"users/{encode_path_param(user_id)}/sessions",
            method="GET",
            params={
                "from": from_,
                "take": take,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListUserSessionsPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListUserSessionsPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.sessions
                _parsed_next = _parsed_response.next
                _has_next = _parsed_next is not None and _parsed_next != ""

                async def _get_next():
                    return await self.list(
                        user_id,
                        from_=_parsed_next,
                        take=take,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Delete all sessions for a user.

        Parameters
        ----------
        user_id : str
            ID of the user to get sessions for

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"users/{encode_path_param(user_id)}/sessions",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class LogSecurityContext(UniversalBaseModel):
    """
    Information about security-related signals.
    """

    ja_3: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="ja3"),
        pydantic.Field(alias="ja3", description="JA3 fingerprint value."),
    ] = None
    """
    JA3 fingerprint value.
    """

    ja_4: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="ja4"),
        pydantic.Field(alias="ja4", description="JA4 fingerprint value."),
    ] = None
    """
    JA4 fingerprint value.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt

ConnectionAdminAccessTokenExpiresInGoogleApps = dt.datetime
"""
Expiration timestamp for the `admin_access_token` in ISO 8601 format. Auth0 uses this value to determine when to refresh the token.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .connection_assertion_decryption_settings import ConnectionAssertionDecryptionSettings
from .connection_attributes import ConnectionAttributes
from .connection_authentication_methods import ConnectionAuthenticationMethods
from .connection_custom_scripts import ConnectionCustomScripts
from .connection_dpop_signing_alg_enum import ConnectionDpopSigningAlgEnum
from .connection_gateway_authentication import ConnectionGatewayAuthentication
from .connection_id_token_session_expiry_supported import ConnectionIdTokenSessionExpirySupported
from .connection_id_token_signed_response_algs import ConnectionIdTokenSignedResponseAlgs
from .connection_identifier_precedence_enum import ConnectionIdentifierPrecedenceEnum
from .connection_passkey_options import ConnectionPasskeyOptions
from .connection_password_complexity_options import ConnectionPasswordComplexityOptions
from .connection_password_dictionary_options import ConnectionPasswordDictionaryOptions
from .connection_password_history_options import ConnectionPasswordHistoryOptions
from .connection_password_no_personal_info_options import ConnectionPasswordNoPersonalInfoOptions
from .connection_password_options import ConnectionPasswordOptions
from .connection_password_policy_enum import ConnectionPasswordPolicyEnum
from .connection_set_user_root_attributes_enum import ConnectionSetUserRootAttributesEnum
from .connection_token_endpoint_auth_method_enum import ConnectionTokenEndpointAuthMethodEnum
from .connection_token_endpoint_auth_signing_alg_enum import ConnectionTokenEndpointAuthSigningAlgEnum
from .connection_token_endpoint_jwtca_aud_format_enum_oidc import ConnectionTokenEndpointJwtcaAudFormatEnumOidc
from .connection_upstream_params import ConnectionUpstreamParams
from .connection_validation_options import ConnectionValidationOptions
from .connections_discovery_url import ConnectionsDiscoveryUrl
from .connections_oidc_metadata import ConnectionsOidcMetadata


class UpdateConnectionOptions(UniversalBaseModel):
    """
    The connection's options (depend on the connection strategy). To update these options, the `update:connections_options` scope must be present. To verify your changes, also include the `read:connections_options` scope. If this scope is not specified, you will not be able to review the updated object.
    """

    validation: typing.Optional[ConnectionValidationOptions] = None
    non_persistent_attrs: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    An array of user fields that should not be stored in the Auth0 database (https://auth0.com/docs/security/data-security/denylist)
    """

    precedence: typing.Optional[typing.List[ConnectionIdentifierPrecedenceEnum]] = pydantic.Field(default=None)
    """
    Order of precedence for attribute types. If the property is not specified, the default precedence of attributes will be used.
    """

    attributes: typing.Optional[ConnectionAttributes] = None
    enable_script_context: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Set to true to inject context into custom DB scripts (warning: cannot be disabled once enabled)
    """

    enabled_database_customization: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="enabledDatabaseCustomization"),
        pydantic.Field(alias="enabledDatabaseCustomization", description="Set to true to use a legacy user store"),
    ] = None
    """
    Set to true to use a legacy user store
    """

    import_mode: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enable this if you have a legacy user store and you want to gradually migrate those users to the Auth0 user store
    """

    configuration: typing.Optional[typing.Dict[str, typing.Optional[str]]] = pydantic.Field(default=None)
    """
    Stores encrypted string only configurations for connections
    """

    custom_scripts: typing_extensions.Annotated[
        typing.Optional[ConnectionCustomScripts],
        FieldMetadata(alias="customScripts"),
        pydantic.Field(alias="customScripts"),
    ] = None
    authentication_methods: typing.Optional[ConnectionAuthenticationMethods] = None
    passkey_options: typing.Optional[ConnectionPasskeyOptions] = None
    password_policy: typing_extensions.Annotated[
        typing.Optional[ConnectionPasswordPolicyEnum],
        FieldMetadata(alias="passwordPolicy"),
        pydantic.Field(alias="passwordPolicy"),
    ] = None
    password_complexity_options: typing.Optional[ConnectionPasswordComplexityOptions] = None
    password_history: typing.Optional[ConnectionPasswordHistoryOptions] = None
    password_no_personal_info: typing.Optional[ConnectionPasswordNoPersonalInfoOptions] = None
    password_dictionary: typing.Optional[ConnectionPasswordDictionaryOptions] = None
    api_enable_users: typing.Optional[bool] = None
    api_enable_groups: typing.Optional[bool] = None
    basic_profile: typing.Optional[bool] = None
    ext_admin: typing.Optional[bool] = None
    ext_is_suspended: typing.Optional[bool] = None
    ext_agreed_terms: typing.Optional[bool] = None
    ext_groups: typing.Optional[bool] = None
    ext_assigned_plans: typing.Optional[bool] = None
    ext_profile: typing.Optional[bool] = None
    disable_self_service_change_password: typing.Optional[bool] = None
    upstream_params: typing.Optional[ConnectionUpstreamParams] = None
    set_user_root_attributes: typing.Optional[ConnectionSetUserRootAttributesEnum] = None
    gateway_authentication: typing.Optional[ConnectionGatewayAuthentication] = None
    password_options: typing.Optional[ConnectionPasswordOptions] = None
    assertion_decryption_settings: typing.Optional[ConnectionAssertionDecryptionSettings] = None
    id_token_signed_response_algs: typing.Optional[ConnectionIdTokenSignedResponseAlgs] = None
    dpop_signing_alg: typing.Optional[ConnectionDpopSigningAlgEnum] = None
    token_endpoint_auth_method: typing.Optional[ConnectionTokenEndpointAuthMethodEnum] = None
    token_endpoint_auth_signing_alg: typing.Optional[ConnectionTokenEndpointAuthSigningAlgEnum] = None
    token_endpoint_jwtca_aud_format: typing.Optional[ConnectionTokenEndpointJwtcaAudFormatEnumOidc] = None
    id_token_session_expiry_supported: typing.Optional[ConnectionIdTokenSessionExpirySupported] = None
    discovery_url: typing.Optional[ConnectionsDiscoveryUrl] = None
    oidc_metadata: typing.Optional[ConnectionsOidcMetadata] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .connection_sha_1_thumbprint import ConnectionSha1Thumbprint

ConnectionThumbprintsSaml = typing.List[ConnectionSha1Thumbprint]
"""
SHA-1 thumbprints (fingerprints) of the identity provider's signing certificates. Automatically computed from signingCert during connection creation. Each thumbprint must be a 40-character hexadecimal string.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

SelfServiceProfileAllowedStrategyEnum = typing.Union[
    typing.Literal[
        "oidc",
        "samlp",
        "waad",
        "google-apps",
        "adfs",
        "okta",
        "auth0-samlp",
        "okta-samlp",
        "keycloak-samlp",
        "pingfederate",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import encode_path_param
from ..core.pagination import AsyncPager, SyncPager
from ..core.parse_error import ParsingError
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_request_error import BadRequestError
from ..errors.forbidden_error import ForbiddenError
from ..errors.not_found_error import NotFoundError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.get_log_response_content import GetLogResponseContent
from ..types.list_log_offset_paginated_response_content import ListLogOffsetPaginatedResponseContent
from ..types.log import Log
from pydantic import ValidationError


class RawLogsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        sort: typing.Optional[str] = None,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        include_totals: typing.Optional[bool] = True,
        search: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Log, ListLogOffsetPaginatedResponseContent]:
        """
        Retrieve log entries that match the specified search criteria (or all log entries if no criteria specified).

        Set custom search criteria using the `q` parameter, or search from a specific log ID (_"search from checkpoint"_).

        For more information on all possible event types, their respective acronyms, and descriptions, see [Log Event Type Codes](https://auth0.com/docs/logs/log-event-type-codes).

        **To set custom search criteria, use the following parameters:**

        - **q:** Search Criteria using [Query String Syntax](https://auth0.com/docs/logs/log-search-query-syntax)
        - **page:** Page index of the results to return. First page is 0.
        - **per_page:** Number of results per page.
        - **sort:** Field to use for sorting appended with `:1` for ascending and `:-1` for descending. e.g. `date:-1`
        - **fields:** Comma-separated list of fields to include or exclude (depending on include_fields) from the result, empty to retrieve all fields.
        - **include_fields:** Whether specified fields are to be included (true) or excluded (false).
        - **include_totals:** Return results inside an object that contains the total result count (true) or as a direct array of results (false, default). **Deprecated:** this field is deprecated and should be removed from use. See [Search Engine V3 Breaking Changes](https://auth0.com/docs/product-lifecycle/deprecations-and-migrations/migrate-to-tenant-log-search-v3#pagination)

        For more information on the list of fields that can be used in `fields` and `sort`, see [Searchable Fields](https://auth0.com/docs/logs/log-search-query-syntax#searchable-fields).

        Auth0 [limits the number of logs](https://auth0.com/docs/logs/retrieve-log-events-using-mgmt-api#limitations) you can return by search criteria to 100 logs per request. Furthermore, you may paginate only through 1,000 search results. If you exceed this threshold, please redefine your search or use the [get logs by checkpoint method](https://auth0.com/docs/logs/retrieve-log-events-using-mgmt-api#retrieve-logs-by-checkpoint).

        **To search from a checkpoint log ID, use the following parameters:**

        - **from:** Log Event ID from which to start retrieving logs. You can limit the number of logs returned using the `take` parameter. If you use `from` at the same time as `q`, `from` takes precedence and `q` is ignored.
        - **take:** Number of entries to retrieve when using the `from` parameter.

        **Important:** When fetching logs from a checkpoint log ID, any parameter other than `from` and `take` will be ignored, and date ordering is not guaranteed.

        Parameters
        ----------
        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
             Number of results per page. Paging is disabled if parameter not sent. Default: <code>50</code>. Max value: <code>100</code>

        sort : typing.Optional[str]
            Field to use for sorting appended with <code>:1</code>  for ascending and <code>:-1</code> for descending. e.g. <code>date:-1</code>

        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for <code>include_fields</code>) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (<code>true</code>) or excluded (<code>false</code>)

        include_totals : typing.Optional[bool]
            Return results as an array when false (default). Return results inside an object that also contains a total result count when true.

        search : typing.Optional[str]
            Retrieves logs that match the specified search criteria. This parameter can be combined with all the others in the /api/logs endpoint but is specified separately for clarity.
            If no fields are provided a case insensitive 'starts with' search is performed on all of the following fields: client_name, connection, user_name. Otherwise, you can specify multiple fields and specify the search using the %field%:%search%, for example: application:node user:"John@contoso.com".
            Values specified without quotes are matched using a case insensitive 'starts with' search. If quotes are used a case insensitve exact search is used. If multiple fields are used, the AND operator is used to join the clauses.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Log, ListLogOffsetPaginatedResponseContent]
            Logs successfully retrieved.
        """
        page = page if page is not None else 0

        _response = self._client_wrapper.httpx_client.request(
            "logs",
            method="GET",
            params={
                "page": page,
                "per_page": per_page,
                "sort": sort,
                "fields": fields,
                "include_fields": include_fields,
                "include_totals": include_totals,
                "search": search,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListLogOffsetPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListLogOffsetPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.logs
                _has_next = len(_items or []) > 0
                _get_next = lambda: self.list(
                    page=page + 1,
                    per_page=per_page,
                    sort=sort,
                    fields=fields,
                    include_fields=include_fields,
                    include_totals=include_totals,
                    search=search,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetLogResponseContent]:
        """
        Retrieve an individual log event.

        Parameters
        ----------
        id : str
            log_id of the log to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetLogResponseContent]
            Log successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"logs/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetLogResponseContent,
                    parse_obj_as(
                        type_=GetLogResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawLogsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        sort: typing.Optional[str] = None,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        include_totals: typing.Optional[bool] = True,
        search: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Log, ListLogOffsetPaginatedResponseContent]:
        """
        Retrieve log entries that match the specified search criteria (or all log entries if no criteria specified).

        Set custom search criteria using the `q` parameter, or search from a specific log ID (_"search from checkpoint"_).

        For more information on all possible event types, their respective acronyms, and descriptions, see [Log Event Type Codes](https://auth0.com/docs/logs/log-event-type-codes).

        **To set custom search criteria, use the following parameters:**

        - **q:** Search Criteria using [Query String Syntax](https://auth0.com/docs/logs/log-search-query-syntax)
        - **page:** Page index of the results to return. First page is 0.
        - **per_page:** Number of results per page.
        - **sort:** Field to use for sorting appended with `:1` for ascending and `:-1` for descending. e.g. `date:-1`
        - **fields:** Comma-separated list of fields to include or exclude (depending on include_fields) from the result, empty to retrieve all fields.
        - **include_fields:** Whether specified fields are to be included (true) or excluded (false).
        - **include_totals:** Return results inside an object that contains the total result count (true) or as a direct array of results (false, default). **Deprecated:** this field is deprecated and should be removed from use. See [Search Engine V3 Breaking Changes](https://auth0.com/docs/product-lifecycle/deprecations-and-migrations/migrate-to-tenant-log-search-v3#pagination)

        For more information on the list of fields that can be used in `fields` and `sort`, see [Searchable Fields](https://auth0.com/docs/logs/log-search-query-syntax#searchable-fields).

        Auth0 [limits the number of logs](https://auth0.com/docs/logs/retrieve-log-events-using-mgmt-api#limitations) you can return by search criteria to 100 logs per request. Furthermore, you may paginate only through 1,000 search results. If you exceed this threshold, please redefine your search or use the [get logs by checkpoint method](https://auth0.com/docs/logs/retrieve-log-events-using-mgmt-api#retrieve-logs-by-checkpoint).

        **To search from a checkpoint log ID, use the following parameters:**

        - **from:** Log Event ID from which to start retrieving logs. You can limit the number of logs returned using the `take` parameter. If you use `from` at the same time as `q`, `from` takes precedence and `q` is ignored.
        - **take:** Number of entries to retrieve when using the `from` parameter.

        **Important:** When fetching logs from a checkpoint log ID, any parameter other than `from` and `take` will be ignored, and date ordering is not guaranteed.

        Parameters
        ----------
        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
             Number of results per page. Paging is disabled if parameter not sent. Default: <code>50</code>. Max value: <code>100</code>

        sort : typing.Optional[str]
            Field to use for sorting appended with <code>:1</code>  for ascending and <code>:-1</code> for descending. e.g. <code>date:-1</code>

        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for <code>include_fields</code>) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (<code>true</code>) or excluded (<code>false</code>)

        include_totals : typing.Optional[bool]
            Return results as an array when false (default). Return results inside an object that also contains a total result count when true.

        search : typing.Optional[str]
            Retrieves logs that match the specified search criteria. This parameter can be combined with all the others in the /api/logs endpoint but is specified separately for clarity.
            If no fields are provided a case insensitive 'starts with' search is performed on all of the following fields: client_name, connection, user_name. Otherwise, you can specify multiple fields and specify the search using the %field%:%search%, for example: application:node user:"John@contoso.com".
            Values specified without quotes are matched using a case insensitive 'starts with' search. If quotes are used a case insensitve exact search is used. If multiple fields are used, the AND operator is used to join the clauses.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Log, ListLogOffsetPaginatedResponseContent]
            Logs successfully retrieved.
        """
        page = page if page is not None else 0

        _response = await self._client_wrapper.httpx_client.request(
            "logs",
            method="GET",
            params={
                "page": page,
                "per_page": per_page,
                "sort": sort,
                "fields": fields,
                "include_fields": include_fields,
                "include_totals": include_totals,
                "search": search,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListLogOffsetPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListLogOffsetPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.logs
                _has_next = len(_items or []) > 0

                async def _get_next():
                    return await self.list(
                        page=page + 1,
                        per_page=per_page,
                        sort=sort,
                        fields=fields,
                        include_fields=include_fields,
                        include_totals=include_totals,
                        search=search,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetLogResponseContent]:
        """
        Retrieve an individual log event.

        Parameters
        ----------
        id : str
            log_id of the log to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetLogResponseContent]
            Log successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"logs/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetLogResponseContent,
                    parse_obj_as(
                        type_=GetLogResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawDuoClient, RawDuoClient

if typing.TYPE_CHECKING:
    from .settings.client import AsyncSettingsClient, SettingsClient


class DuoClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawDuoClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._settings: typing.Optional[SettingsClient] = None

    @property
    def with_raw_response(self) -> RawDuoClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawDuoClient
        """
        return self._raw_client

    @property
    def settings(self):
        if self._settings is None:
            from .settings.client import SettingsClient  # noqa: E402

            self._settings = SettingsClient(client_wrapper=self._client_wrapper)
        return self._settings


class AsyncDuoClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawDuoClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._settings: typing.Optional[AsyncSettingsClient] = None

    @property
    def with_raw_response(self) -> AsyncRawDuoClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawDuoClient
        """
        return self._raw_client

    @property
    def settings(self):
        if self._settings is None:
            from .settings.client import AsyncSettingsClient  # noqa: E402

            self._settings = AsyncSettingsClient(client_wrapper=self._client_wrapper)
        return self._settings



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .event_stream_cloud_event_org_created_object_branding import EventStreamCloudEventOrgCreatedObjectBranding
from .event_stream_cloud_event_org_created_object_metadata import EventStreamCloudEventOrgCreatedObjectMetadata


class EventStreamCloudEventOrgCreatedObject(UniversalBaseModel):
    """
    The event content.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The human-readable identifier for the organization that will be used by end-users to direct them to their organization in your application..
    """

    id: str = pydantic.Field()
    """
    ID of the organization.
    """

    display_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    If set, the name that will be displayed to end-users for this organization in any interaction with them.
    """

    metadata: typing.Optional[EventStreamCloudEventOrgCreatedObjectMetadata] = None
    branding: typing.Optional[EventStreamCloudEventOrgCreatedObjectBranding] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionOptionsIdpInitiatedClientProtocolEnumSaml = typing.Union[typing.Literal["oidc", "samlp", "wsfed"], typing.Any]



import unittest
import requests
from fnmatch import fnmatch
from unittest import mock
from unittest.mock import ANY

from cryptography.hazmat.primitives import asymmetric, serialization

from auth0.authentication.exceptions import RateLimitError
from auth0.authentication.get_token import GetToken


def get_private_key():
    private_key = asymmetric.rsa.generate_private_key(
        public_exponent=65537,
        key_size=2048,
    )
    return private_key.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.PKCS8,
        encryption_algorithm=serialization.NoEncryption(),
    )


class TestGetToken(unittest.TestCase):
    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_authorization_code(self, mock_post):
        g = GetToken("my.domain.com", "cid", client_secret="clsec")

        g.authorization_code(
            code="cd",
            grant_type="gt",
            redirect_uri="idt",
        )

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/token")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "client_secret": "clsec",
                "code": "cd",
                "grant_type": "gt",
                "redirect_uri": "idt",
            },
        )

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_authorization_code_with_client_assertion(self, mock_post):
        g = GetToken(
            "my.domain.com", "cid", client_assertion_signing_key=get_private_key()
        )

        g.authorization_code(code="cd", grant_type="gt", redirect_uri="idt")

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/token")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "client_assertion": ANY,
                "client_assertion_type": "urn:ietf:params:oauth:client-assertion-type:jwt-bearer",
                "code": "cd",
                "grant_type": "gt",
                "redirect_uri": "idt",
            },
        )

        self.assertTrue(fnmatch(kwargs["data"]["client_assertion"], "*.*.*"))

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_authorization_code_pkce(self, mock_post):
        g = GetToken("my.domain.com", "cid")

        g.authorization_code_pkce(
            code_verifier="cdver",
            code="cd",
            grant_type="gt",
            redirect_uri="idt",
        )

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/token")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "code_verifier": "cdver",
                "code": "cd",
                "grant_type": "gt",
                "redirect_uri": "idt",
            },
        )

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_client_credentials(self, mock_post):
        g = GetToken("my.domain.com", "cid", client_secret="clsec")

        g.client_credentials(audience="aud", grant_type="gt")

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/token")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "client_secret": "clsec",
                "audience": "aud",
                "grant_type": "gt",
                "organization": None,
            },
        )

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_client_credentials_with_client_assertion(self, mock_post):
        g = GetToken(
            "my.domain.com", "cid", client_assertion_signing_key=get_private_key()
        )

        g.client_credentials("aud", grant_type="gt")

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/token")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "client_assertion": ANY,
                "client_assertion_type": "urn:ietf:params:oauth:client-assertion-type:jwt-bearer",
                "audience": "aud",
                "grant_type": "gt",
                "organization": None,
            },
        )

        self.assertTrue(fnmatch(kwargs["data"]["client_assertion"], "*.*.*"))

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_client_credentials_with_organization(self, mock_post):
        g = GetToken("my.domain.com", "cid", client_secret="clsec")

        g.client_credentials("aud", organization="my-org")

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/token")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "grant_type": "client_credentials",
                "client_secret": "clsec",
                "audience": "aud",
                "organization": "my-org",
            },
        )

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_login(self, mock_post):
        g = GetToken("my.domain.com", "cid", client_secret="clsec")

        g.login(
            username="usrnm",
            password="pswd",
            scope="http://test.com/api",
            realm="rlm",
            audience="aud",
            grant_type="gt",
        )

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/token")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "client_secret": "clsec",
                "username": "usrnm",
                "password": "pswd",
                "scope": "http://test.com/api",
                "realm": "rlm",
                "audience": "aud",
                "grant_type": "gt",
            },
        )

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_login_simple(self, mock_post):
        g = GetToken("my.domain.com", "cid", client_secret="clsec")

        g.login(
            username="usrnm",
            password="pswd",
        )

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/token")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "client_secret": "clsec",
                "username": "usrnm",
                "password": "pswd",
                "realm": None,
                "scope": None,
                "audience": None,
                "grant_type": "http://auth0.com/oauth/grant-type/password-realm",
            },
        )

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_login_with_forwarded_for(self, mock_post):
        g = GetToken("my.domain.com", "cid", client_secret="clsec")

        g.login(username="usrnm", password="pswd", forwarded_for="192.168.0.1")

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/token")
        self.assertEqual(
            kwargs["headers"],
            {
                "auth0-forwarded-for": "192.168.0.1",
            },
        )

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_refresh_token(self, mock_post):
        g = GetToken("my.domain.com", "cid", client_secret="clsec")

        g.refresh_token(
            refresh_token="rt",
            grant_type="gt",
            scope="s",
        )

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/token")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "client_secret": "clsec",
                "refresh_token": "rt",
                "grant_type": "gt",
                "scope": "s",
            },
        )

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_passwordless_login_with_sms(self, mock_post):
        g = GetToken("my.domain.com", "cid", client_secret="csec")

        g.passwordless_login(
            username="123456",
            otp="abcd",
            realm="sms",
            audience="aud",
            scope="openid",
        )

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/token")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "client_secret": "csec",
                "realm": "sms",
                "grant_type": "http://auth0.com/oauth/grant-type/passwordless/otp",
                "username": "123456",
                "otp": "abcd",
                "audience": "aud",
                "scope": "openid",
            },
        )

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_passwordless_login_with_email(self, mock_post):
        g = GetToken("my.domain.com", "cid", client_secret="csec")

        g.passwordless_login(
            username="a@b.c",
            otp="abcd",
            realm="email",
            audience="aud",
            scope="openid",
        )

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/token")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "client_secret": "csec",
                "realm": "email",
                "grant_type": "http://auth0.com/oauth/grant-type/passwordless/otp",
                "username": "a@b.c",
                "otp": "abcd",
                "audience": "aud",
                "scope": "openid",
            },
        )

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_backchannel_login(self, mock_post):
        g = GetToken("my.domain.com", "cid", client_secret="csec")

        g.backchannel_login(
            auth_req_id="reqid",
            grant_type="urn:openid:params:grant-type:ciba",
        )

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/token")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "client_secret": "csec",
                "auth_req_id": "reqid",
                "grant_type": "urn:openid:params:grant-type:ciba",
            },
        )

    @mock.patch("requests.request")
    def test_backchannel_login_headers_on_slow_down(self, mock_requests_request):
        response = requests.Response()
        response.status_code = 429
        response.headers = {"Retry-After": "100"}
        response._content = b'{"error":"slow_down"}'
        mock_requests_request.return_value = response

        g = GetToken("my.domain.com", "cid", client_secret="csec")

        with self.assertRaises(RateLimitError) as context:
            g.backchannel_login(
                auth_req_id="reqid",
                grant_type="urn:openid:params:grant-type:ciba",
            )
        self.assertEqual(context.exception.headers["Retry-After"], "100")
        self.assertEqual(context.exception.status_code, 429)

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_connection_login(self, mock_post):
        g = GetToken("my.domain.com", "cid", client_secret="csec")

        g.access_token_for_connection(
            grant_type="urn:auth0:params:oauth:grant-type:token-exchange:federated-connection-access-token",
            subject_token_type="urn:ietf:params:oauth:token-type:refresh_token",
            subject_token="refid",
            requested_token_type="http://auth0.com/oauth/token-type/federated-connection-access-token",
            connection="google-oauth2"
        )

        args, kwargs = mock_post.call_args

        print(kwargs["data"])

        self.assertEqual(args[0], "https://my.domain.com/oauth/token")
        self.assertEqual(
            kwargs["data"],
            {
                "grant_type": "urn:auth0:params:oauth:grant-type:token-exchange:federated-connection-access-token",
                "client_id": "cid",
                "client_secret": "csec",
                "subject_token_type": "urn:ietf:params:oauth:token-type:refresh_token",
                "subject_token": "refid",
                "requested_token_type": "http://auth0.com/oauth/token-type/federated-connection-access-token",
                "connection": "google-oauth2"
            },
        )

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_connection_login_with_login_hint(self, mock_post):
        g = GetToken("my.domain.com", "cid", client_secret="csec")

        g.access_token_for_connection(
            subject_token_type="urn:ietf:params:oauth:token-type:refresh_token",
            subject_token="refid",
            requested_token_type="http://auth0.com/oauth/token-type/federated-connection-access-token",
            connection="google-oauth2",
            login_hint="john.doe@example.com"
        )

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/token")
        self.assertEqual(
            kwargs["data"],
            {
                "grant_type": "urn:auth0:params:oauth:grant-type:token-exchange:federated-connection-access-token",
                "client_id": "cid",
                "client_secret": "csec",
                "subject_token_type": "urn:ietf:params:oauth:token-type:refresh_token",
                "subject_token": "refid",
                "requested_token_type": "http://auth0.com/oauth/token-type/federated-connection-access-token",
                "connection": "google-oauth2",
                "login_hint": "john.doe@example.com"
            },
        )


# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .log_stream_mixpanel_region_enum import LogStreamMixpanelRegionEnum


class LogStreamMixpanelSink(UniversalBaseModel):
    mixpanel_region: typing_extensions.Annotated[
        LogStreamMixpanelRegionEnum, FieldMetadata(alias="mixpanelRegion"), pydantic.Field(alias="mixpanelRegion")
    ]
    mixpanel_project_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="mixpanelProjectId"),
        pydantic.Field(alias="mixpanelProjectId", description="Mixpanel Project Id"),
    ]
    """
    Mixpanel Project Id
    """

    mixpanel_service_account_username: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="mixpanelServiceAccountUsername"),
        pydantic.Field(alias="mixpanelServiceAccountUsername", description="Mixpanel Service Account Username"),
    ]
    """
    Mixpanel Service Account Username
    """

    mixpanel_service_account_password: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="mixpanelServiceAccountPassword"),
        pydantic.Field(alias="mixpanelServiceAccountPassword", description="Mixpanel Service Account Password"),
    ]
    """
    Mixpanel Service Account Password
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count


def test_keys_signing_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "keys.signing.list_.0"
    client = get_client(test_id)
    client.keys.signing.list()
    verify_request_count(test_id, "GET", "/keys/signing", None, 1)


def test_keys_signing_rotate() -> None:
    """Test rotate endpoint with WireMock"""
    test_id = "keys.signing.rotate.0"
    client = get_client(test_id)
    client.keys.signing.rotate()
    verify_request_count(test_id, "POST", "/keys/signing/rotate", None, 1)


def test_keys_signing_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "keys.signing.get.0"
    client = get_client(test_id)
    client.keys.signing.get(
        kid="kid",
    )
    verify_request_count(test_id, "GET", "/keys/signing/kid", None, 1)


def test_keys_signing_revoke() -> None:
    """Test revoke endpoint with WireMock"""
    test_id = "keys.signing.revoke.0"
    client = get_client(test_id)
    client.keys.signing.revoke(
        kid="kid",
    )
    verify_request_count(test_id, "PUT", "/keys/signing/kid/revoke", None, 1)



# This file was auto-generated by Fern from our API Definition.

from .connection_scope_array import ConnectionScopeArray

ConnectionScopeGoogleOAuth2 = ConnectionScopeArray
"""
Array of OAuth 2.0 scopes requested during Google authentication.
"""



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .event_stream_cloud_event_connection_updated_object_4_options_set_user_root_attributes_enum import (
    EventStreamCloudEventConnectionUpdatedObject4OptionsSetUserRootAttributesEnum,
)
from .event_stream_cloud_event_connection_updated_object_4_options_should_trust_email_verified_connection_enum import (
    EventStreamCloudEventConnectionUpdatedObject4OptionsShouldTrustEmailVerifiedConnectionEnum,
)
from .event_stream_cloud_event_connection_updated_object_4_options_upstream_params import (
    EventStreamCloudEventConnectionUpdatedObject4OptionsUpstreamParams,
)


class EventStreamCloudEventConnectionUpdatedObject4Options(UniversalBaseModel):
    """
    Options for the 'adfs' connection
    """

    adfs_server: typing.Optional[str] = pydantic.Field(default=None)
    """
    ADFS federation metadata host or XML URL used to discover WS-Fed endpoints and certificates. Errors if adfs_server and fedMetadataXml are both absent.
    """

    cert_rollover_notification: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Timestamp of the last certificate expiring soon notification.
    """

    domain_aliases: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Email domains associated with this connection for Home Realm Discovery (HRD). When a user's email matches one of these domains, they are automatically routed to this connection during authentication.
    """

    entity_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="entityId"),
        pydantic.Field(
            alias="entityId",
            description="The entity identifier (Issuer) for the ADFS Service Provider. When not provided, defaults to 'urn:auth0:{tenant}:{connection}'.",
        ),
    ] = None
    """
    The entity identifier (Issuer) for the ADFS Service Provider. When not provided, defaults to 'urn:auth0:{tenant}:{connection}'.
    """

    fed_metadata_xml: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="fedMetadataXml"),
        pydantic.Field(
            alias="fedMetadataXml",
            description="Inline XML alternative to 'adfs_server'. Cannot be set together with 'adfs_server'.",
        ),
    ] = None
    """
    Inline XML alternative to 'adfs_server'. Cannot be set together with 'adfs_server'.
    """

    icon_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL for the connection icon displayed in Auth0 login pages. Accepts HTTPS URLs. Used for visual branding in authentication flows.
    """

    non_persistent_attrs: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    An array of user fields that should not be stored in the Auth0 database (https://auth0.com/docs/security/data-security/denylist)
    """

    prev_thumbprints: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Array of certificate thumbprints (SHA-128/SHA-256/SHA-512 hex hashes) for validating SAML signatures. Used with WS-Federation protocol. Maximum 20 thumbprints. Each thumbprint must be a hexadecimal string.
    """

    set_user_root_attributes: typing.Optional[
        EventStreamCloudEventConnectionUpdatedObject4OptionsSetUserRootAttributesEnum
    ] = None
    should_trust_email_verified_connection: typing.Optional[
        EventStreamCloudEventConnectionUpdatedObject4OptionsShouldTrustEmailVerifiedConnectionEnum
    ] = None
    sign_in_endpoint: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="signInEndpoint"),
        pydantic.Field(
            alias="signInEndpoint",
            description="Passive Requestor (WS-Fed) sign-in endpoint discovered from metadata or provided explicitly.",
        ),
    ] = None
    """
    Passive Requestor (WS-Fed) sign-in endpoint discovered from metadata or provided explicitly.
    """

    tenant_domain: typing.Optional[str] = pydantic.Field(default=None)
    """
    Tenant domain
    """

    thumbprints: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Array of certificate thumbprints (SHA-128/SHA-256/SHA-512 hex hashes) for validating SAML signatures. Used with WS-Federation protocol. Maximum 20 thumbprints. Each thumbprint must be a hexadecimal string.
    """

    upstream_params: typing.Optional[EventStreamCloudEventConnectionUpdatedObject4OptionsUpstreamParams] = None
    user_id_attribute: typing.Optional[str] = pydantic.Field(default=None)
    """
    Custom ADFS claim to use as the unique user identifier. When provided, this attribute is prepended to the default user_id mapping list with highest priority. Accepts a string (single ADFS claim name).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .get_universal_login_template import GetUniversalLoginTemplate

GetUniversalLoginTemplateResponseContent = typing.Union[GetUniversalLoginTemplate, str]



# This file was auto-generated by Fern from our API Definition.

ConnectionEntityIdSaml = str
"""
The entity identifier (Issuer) for the SAML Service Provider. When not provided, defaults to 'urn:auth0:{tenant}:{connection}'. This value is included in SAML AuthnRequest messages sent to the identity provider.
"""



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawAttackProtectionClient, RawAttackProtectionClient

if typing.TYPE_CHECKING:
    from .bot_detection.client import AsyncBotDetectionClient, BotDetectionClient
    from .breached_password_detection.client import (
        AsyncBreachedPasswordDetectionClient,
        BreachedPasswordDetectionClient,
    )
    from .brute_force_protection.client import AsyncBruteForceProtectionClient, BruteForceProtectionClient
    from .captcha.client import AsyncCaptchaClient, CaptchaClient
    from .phone_provider_protection.client import AsyncPhoneProviderProtectionClient, PhoneProviderProtectionClient
    from .suspicious_ip_throttling.client import AsyncSuspiciousIpThrottlingClient, SuspiciousIpThrottlingClient


class AttackProtectionClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAttackProtectionClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._bot_detection: typing.Optional[BotDetectionClient] = None
        self._breached_password_detection: typing.Optional[BreachedPasswordDetectionClient] = None
        self._brute_force_protection: typing.Optional[BruteForceProtectionClient] = None
        self._captcha: typing.Optional[CaptchaClient] = None
        self._phone_provider_protection: typing.Optional[PhoneProviderProtectionClient] = None
        self._suspicious_ip_throttling: typing.Optional[SuspiciousIpThrottlingClient] = None

    @property
    def with_raw_response(self) -> RawAttackProtectionClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAttackProtectionClient
        """
        return self._raw_client

    @property
    def bot_detection(self):
        if self._bot_detection is None:
            from .bot_detection.client import BotDetectionClient  # noqa: E402

            self._bot_detection = BotDetectionClient(client_wrapper=self._client_wrapper)
        return self._bot_detection

    @property
    def breached_password_detection(self):
        if self._breached_password_detection is None:
            from .breached_password_detection.client import BreachedPasswordDetectionClient  # noqa: E402

            self._breached_password_detection = BreachedPasswordDetectionClient(client_wrapper=self._client_wrapper)
        return self._breached_password_detection

    @property
    def brute_force_protection(self):
        if self._brute_force_protection is None:
            from .brute_force_protection.client import BruteForceProtectionClient  # noqa: E402

            self._brute_force_protection = BruteForceProtectionClient(client_wrapper=self._client_wrapper)
        return self._brute_force_protection

    @property
    def captcha(self):
        if self._captcha is None:
            from .captcha.client import CaptchaClient  # noqa: E402

            self._captcha = CaptchaClient(client_wrapper=self._client_wrapper)
        return self._captcha

    @property
    def phone_provider_protection(self):
        if self._phone_provider_protection is None:
            from .phone_provider_protection.client import PhoneProviderProtectionClient  # noqa: E402

            self._phone_provider_protection = PhoneProviderProtectionClient(client_wrapper=self._client_wrapper)
        return self._phone_provider_protection

    @property
    def suspicious_ip_throttling(self):
        if self._suspicious_ip_throttling is None:
            from .suspicious_ip_throttling.client import SuspiciousIpThrottlingClient  # noqa: E402

            self._suspicious_ip_throttling = SuspiciousIpThrottlingClient(client_wrapper=self._client_wrapper)
        return self._suspicious_ip_throttling


class AsyncAttackProtectionClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAttackProtectionClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._bot_detection: typing.Optional[AsyncBotDetectionClient] = None
        self._breached_password_detection: typing.Optional[AsyncBreachedPasswordDetectionClient] = None
        self._brute_force_protection: typing.Optional[AsyncBruteForceProtectionClient] = None
        self._captcha: typing.Optional[AsyncCaptchaClient] = None
        self._phone_provider_protection: typing.Optional[AsyncPhoneProviderProtectionClient] = None
        self._suspicious_ip_throttling: typing.Optional[AsyncSuspiciousIpThrottlingClient] = None

    @property
    def with_raw_response(self) -> AsyncRawAttackProtectionClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAttackProtectionClient
        """
        return self._raw_client

    @property
    def bot_detection(self):
        if self._bot_detection is None:
            from .bot_detection.client import AsyncBotDetectionClient  # noqa: E402

            self._bot_detection = AsyncBotDetectionClient(client_wrapper=self._client_wrapper)
        return self._bot_detection

    @property
    def breached_password_detection(self):
        if self._breached_password_detection is None:
            from .breached_password_detection.client import AsyncBreachedPasswordDetectionClient  # noqa: E402

            self._breached_password_detection = AsyncBreachedPasswordDetectionClient(
                client_wrapper=self._client_wrapper
            )
        return self._breached_password_detection

    @property
    def brute_force_protection(self):
        if self._brute_force_protection is None:
            from .brute_force_protection.client import AsyncBruteForceProtectionClient  # noqa: E402

            self._brute_force_protection = AsyncBruteForceProtectionClient(client_wrapper=self._client_wrapper)
        return self._brute_force_protection

    @property
    def captcha(self):
        if self._captcha is None:
            from .captcha.client import AsyncCaptchaClient  # noqa: E402

            self._captcha = AsyncCaptchaClient(client_wrapper=self._client_wrapper)
        return self._captcha

    @property
    def phone_provider_protection(self):
        if self._phone_provider_protection is None:
            from .phone_provider_protection.client import AsyncPhoneProviderProtectionClient  # noqa: E402

            self._phone_provider_protection = AsyncPhoneProviderProtectionClient(client_wrapper=self._client_wrapper)
        return self._phone_provider_protection

    @property
    def suspicious_ip_throttling(self):
        if self._suspicious_ip_throttling is None:
            from .suspicious_ip_throttling.client import AsyncSuspiciousIpThrottlingClient  # noqa: E402

            self._suspicious_ip_throttling = AsyncSuspiciousIpThrottlingClient(client_wrapper=self._client_wrapper)
        return self._suspicious_ip_throttling



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .list_aculs_response_content_item import ListAculsResponseContentItem


class ListAculsOffsetPaginatedResponseContent(UniversalBaseModel):
    configs: typing.Optional[typing.List[ListAculsResponseContentItem]] = None
    start: typing.Optional[float] = pydantic.Field(default=None)
    """
    the index of the first configuration in the response (before filtering)
    """

    limit: typing.Optional[float] = pydantic.Field(default=None)
    """
    the maximum number of configurations shown per page (before filtering)
    """

    total: typing.Optional[float] = pydantic.Field(default=None)
    """
    the total number of configurations on this tenant
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.create_resource_server_response_content import CreateResourceServerResponseContent
from ..types.get_resource_server_response_content import GetResourceServerResponseContent
from ..types.list_resource_server_offset_paginated_response_content import (
    ListResourceServerOffsetPaginatedResponseContent,
)
from ..types.resource_server import ResourceServer
from ..types.resource_server_authorization_policy import ResourceServerAuthorizationPolicy
from ..types.resource_server_consent_policy_enum import ResourceServerConsentPolicyEnum
from ..types.resource_server_proof_of_possession import ResourceServerProofOfPossession
from ..types.resource_server_scope import ResourceServerScope
from ..types.resource_server_subject_type_authorization import ResourceServerSubjectTypeAuthorization
from ..types.resource_server_token_dialect_schema_enum import ResourceServerTokenDialectSchemaEnum
from ..types.resource_server_token_encryption import ResourceServerTokenEncryption
from ..types.signing_algorithm_enum import SigningAlgorithmEnum
from ..types.update_resource_server_response_content import UpdateResourceServerResponseContent
from .raw_client import AsyncRawResourceServersClient, RawResourceServersClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ResourceServersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawResourceServersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawResourceServersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawResourceServersClient
        """
        return self._raw_client

    def list(
        self,
        *,
        identifiers: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ResourceServer, ListResourceServerOffsetPaginatedResponseContent]:
        """
        Retrieve details of all APIs associated with your tenant.

        Parameters
        ----------
        identifiers : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            An optional filter on the resource server identifier. Must be URL encoded and may be specified multiple times (max 10).<br /><b>e.g.</b> <i>../resource-servers?identifiers=id1&identifiers=id2</i>

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ResourceServer, ListResourceServerOffsetPaginatedResponseContent]
            Resource servers successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.resource_servers.list(
            identifiers=["identifiers"],
            page=1,
            per_page=1,
            include_totals=True,
            include_fields=True,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            identifiers=identifiers,
            page=page,
            per_page=per_page,
            include_totals=include_totals,
            include_fields=include_fields,
            request_options=request_options,
        )

    def create(
        self,
        *,
        identifier: str,
        name: typing.Optional[str] = OMIT,
        scopes: typing.Optional[typing.Sequence[ResourceServerScope]] = OMIT,
        signing_alg: typing.Optional[SigningAlgorithmEnum] = OMIT,
        signing_secret: typing.Optional[str] = OMIT,
        allow_offline_access: typing.Optional[bool] = OMIT,
        allow_online_access: typing.Optional[bool] = OMIT,
        allow_online_access_with_ephemeral_sessions: typing.Optional[bool] = OMIT,
        token_lifetime: typing.Optional[int] = OMIT,
        token_dialect: typing.Optional[ResourceServerTokenDialectSchemaEnum] = OMIT,
        skip_consent_for_verifiable_first_party_clients: typing.Optional[bool] = OMIT,
        enforce_policies: typing.Optional[bool] = OMIT,
        token_encryption: typing.Optional[ResourceServerTokenEncryption] = OMIT,
        consent_policy: typing.Optional[ResourceServerConsentPolicyEnum] = OMIT,
        authorization_details: typing.Optional[typing.Sequence[typing.Any]] = OMIT,
        proof_of_possession: typing.Optional[ResourceServerProofOfPossession] = OMIT,
        subject_type_authorization: typing.Optional[ResourceServerSubjectTypeAuthorization] = OMIT,
        authorization_policy: typing.Optional[ResourceServerAuthorizationPolicy] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateResourceServerResponseContent:
        """
        Create a new API associated with your tenant. Note that all new APIs must be registered with Auth0. For more information, read <a href="https://www.auth0.com/docs/get-started/apis"> APIs</a>.

        Parameters
        ----------
        identifier : str
            Unique identifier for the API used as the audience parameter on authorization calls. Can not be changed once set.

        name : typing.Optional[str]
            Friendly name for this resource server. Can not contain `<` or `>` characters.

        scopes : typing.Optional[typing.Sequence[ResourceServerScope]]
            List of permissions (scopes) that this API uses.

        signing_alg : typing.Optional[SigningAlgorithmEnum]

        signing_secret : typing.Optional[str]
            Secret used to sign tokens when using symmetric algorithms (HS256).

        allow_offline_access : typing.Optional[bool]
            Whether refresh tokens can be issued for this API (true) or not (false).

        allow_online_access : typing.Optional[bool]
            Whether Online Refresh Tokens can be issued for this API (true) or not (false).

        allow_online_access_with_ephemeral_sessions : typing.Optional[bool]
            Whether Online Refresh Tokens can be issued even when sessions are configured as ephemeral (true) or not (false).

        token_lifetime : typing.Optional[int]
            Expiration value (in seconds) for access tokens issued for this API from the token endpoint.

        token_dialect : typing.Optional[ResourceServerTokenDialectSchemaEnum]

        skip_consent_for_verifiable_first_party_clients : typing.Optional[bool]
            Whether to skip user consent for applications flagged as first party (true) or not (false).

        enforce_policies : typing.Optional[bool]
            Whether to enforce authorization policies (true) or to ignore them (false).

        token_encryption : typing.Optional[ResourceServerTokenEncryption]

        consent_policy : typing.Optional[ResourceServerConsentPolicyEnum]

        authorization_details : typing.Optional[typing.Sequence[typing.Any]]

        proof_of_possession : typing.Optional[ResourceServerProofOfPossession]

        subject_type_authorization : typing.Optional[ResourceServerSubjectTypeAuthorization]

        authorization_policy : typing.Optional[ResourceServerAuthorizationPolicy]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateResourceServerResponseContent
            Resource server successfully created.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.resource_servers.create(
            identifier="identifier",
        )
        """
        _response = self._raw_client.create(
            identifier=identifier,
            name=name,
            scopes=scopes,
            signing_alg=signing_alg,
            signing_secret=signing_secret,
            allow_offline_access=allow_offline_access,
            allow_online_access=allow_online_access,
            allow_online_access_with_ephemeral_sessions=allow_online_access_with_ephemeral_sessions,
            token_lifetime=token_lifetime,
            token_dialect=token_dialect,
            skip_consent_for_verifiable_first_party_clients=skip_consent_for_verifiable_first_party_clients,
            enforce_policies=enforce_policies,
            token_encryption=token_encryption,
            consent_policy=consent_policy,
            authorization_details=authorization_details,
            proof_of_possession=proof_of_possession,
            subject_type_authorization=subject_type_authorization,
            authorization_policy=authorization_policy,
            request_options=request_options,
        )
        return _response.data

    def get(
        self,
        id: str,
        *,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetResourceServerResponseContent:
        """
        Retrieve <a href="https://auth0.com/docs/apis">API</a> details with the given ID.

        Parameters
        ----------
        id : str
            ID or audience of the resource server to retrieve.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetResourceServerResponseContent
            Resource server successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.resource_servers.get(
            id="id",
            include_fields=True,
        )
        """
        _response = self._raw_client.get(id, include_fields=include_fields, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete an existing API by ID. For more information, read <a href="https://www.auth0.com/docs/get-started/apis/api-settings">API Settings</a>.

        Parameters
        ----------
        id : str
            ID or the audience of the resource server to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.resource_servers.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        scopes: typing.Optional[typing.Sequence[ResourceServerScope]] = OMIT,
        signing_alg: typing.Optional[SigningAlgorithmEnum] = OMIT,
        signing_secret: typing.Optional[str] = OMIT,
        skip_consent_for_verifiable_first_party_clients: typing.Optional[bool] = OMIT,
        allow_offline_access: typing.Optional[bool] = OMIT,
        allow_online_access: typing.Optional[bool] = OMIT,
        allow_online_access_with_ephemeral_sessions: typing.Optional[bool] = OMIT,
        token_lifetime: typing.Optional[int] = OMIT,
        token_dialect: typing.Optional[ResourceServerTokenDialectSchemaEnum] = OMIT,
        enforce_policies: typing.Optional[bool] = OMIT,
        token_encryption: typing.Optional[ResourceServerTokenEncryption] = OMIT,
        consent_policy: typing.Optional[ResourceServerConsentPolicyEnum] = OMIT,
        authorization_details: typing.Optional[typing.Sequence[typing.Any]] = OMIT,
        proof_of_possession: typing.Optional[ResourceServerProofOfPossession] = OMIT,
        subject_type_authorization: typing.Optional[ResourceServerSubjectTypeAuthorization] = OMIT,
        authorization_policy: typing.Optional[ResourceServerAuthorizationPolicy] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateResourceServerResponseContent:
        """
        Change an existing API setting by resource server ID. For more information, read <a href="https://www.auth0.com/docs/get-started/apis/api-settings">API Settings</a>.

        Parameters
        ----------
        id : str
            ID or audience of the resource server to update.

        name : typing.Optional[str]
            Friendly name for this resource server. Can not contain `<` or `>` characters.

        scopes : typing.Optional[typing.Sequence[ResourceServerScope]]
            List of permissions (scopes) that this API uses.

        signing_alg : typing.Optional[SigningAlgorithmEnum]

        signing_secret : typing.Optional[str]
            Secret used to sign tokens when using symmetric algorithms (HS256).

        skip_consent_for_verifiable_first_party_clients : typing.Optional[bool]
            Whether to skip user consent for applications flagged as first party (true) or not (false).

        allow_offline_access : typing.Optional[bool]
            Whether refresh tokens can be issued for this API (true) or not (false).

        allow_online_access : typing.Optional[bool]
            Whether Online Refresh Tokens can be issued for this API (true) or not (false).

        allow_online_access_with_ephemeral_sessions : typing.Optional[bool]
            Whether Online Refresh Tokens can be issued even when sessions are configured as ephemeral (true) or not (false).

        token_lifetime : typing.Optional[int]
            Expiration value (in seconds) for access tokens issued for this API from the token endpoint.

        token_dialect : typing.Optional[ResourceServerTokenDialectSchemaEnum]

        enforce_policies : typing.Optional[bool]
            Whether authorization policies are enforced (true) or not enforced (false).

        token_encryption : typing.Optional[ResourceServerTokenEncryption]

        consent_policy : typing.Optional[ResourceServerConsentPolicyEnum]

        authorization_details : typing.Optional[typing.Sequence[typing.Any]]

        proof_of_possession : typing.Optional[ResourceServerProofOfPossession]

        subject_type_authorization : typing.Optional[ResourceServerSubjectTypeAuthorization]

        authorization_policy : typing.Optional[ResourceServerAuthorizationPolicy]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateResourceServerResponseContent
            Resource server successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.resource_servers.update(
            id="id",
        )
        """
        _response = self._raw_client.update(
            id,
            name=name,
            scopes=scopes,
            signing_alg=signing_alg,
            signing_secret=signing_secret,
            skip_consent_for_verifiable_first_party_clients=skip_consent_for_verifiable_first_party_clients,
            allow_offline_access=allow_offline_access,
            allow_online_access=allow_online_access,
            allow_online_access_with_ephemeral_sessions=allow_online_access_with_ephemeral_sessions,
            token_lifetime=token_lifetime,
            token_dialect=token_dialect,
            enforce_policies=enforce_policies,
            token_encryption=token_encryption,
            consent_policy=consent_policy,
            authorization_details=authorization_details,
            proof_of_possession=proof_of_possession,
            subject_type_authorization=subject_type_authorization,
            authorization_policy=authorization_policy,
            request_options=request_options,
        )
        return _response.data


class AsyncResourceServersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawResourceServersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawResourceServersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawResourceServersClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        identifiers: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ResourceServer, ListResourceServerOffsetPaginatedResponseContent]:
        """
        Retrieve details of all APIs associated with your tenant.

        Parameters
        ----------
        identifiers : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            An optional filter on the resource server identifier. Must be URL encoded and may be specified multiple times (max 10).<br /><b>e.g.</b> <i>../resource-servers?identifiers=id1&identifiers=id2</i>

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ResourceServer, ListResourceServerOffsetPaginatedResponseContent]
            Resource servers successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.resource_servers.list(
                identifiers=["identifiers"],
                page=1,
                per_page=1,
                include_totals=True,
                include_fields=True,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            identifiers=identifiers,
            page=page,
            per_page=per_page,
            include_totals=include_totals,
            include_fields=include_fields,
            request_options=request_options,
        )

    async def create(
        self,
        *,
        identifier: str,
        name: typing.Optional[str] = OMIT,
        scopes: typing.Optional[typing.Sequence[ResourceServerScope]] = OMIT,
        signing_alg: typing.Optional[SigningAlgorithmEnum] = OMIT,
        signing_secret: typing.Optional[str] = OMIT,
        allow_offline_access: typing.Optional[bool] = OMIT,
        allow_online_access: typing.Optional[bool] = OMIT,
        allow_online_access_with_ephemeral_sessions: typing.Optional[bool] = OMIT,
        token_lifetime: typing.Optional[int] = OMIT,
        token_dialect: typing.Optional[ResourceServerTokenDialectSchemaEnum] = OMIT,
        skip_consent_for_verifiable_first_party_clients: typing.Optional[bool] = OMIT,
        enforce_policies: typing.Optional[bool] = OMIT,
        token_encryption: typing.Optional[ResourceServerTokenEncryption] = OMIT,
        consent_policy: typing.Optional[ResourceServerConsentPolicyEnum] = OMIT,
        authorization_details: typing.Optional[typing.Sequence[typing.Any]] = OMIT,
        proof_of_possession: typing.Optional[ResourceServerProofOfPossession] = OMIT,
        subject_type_authorization: typing.Optional[ResourceServerSubjectTypeAuthorization] = OMIT,
        authorization_policy: typing.Optional[ResourceServerAuthorizationPolicy] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateResourceServerResponseContent:
        """
        Create a new API associated with your tenant. Note that all new APIs must be registered with Auth0. For more information, read <a href="https://www.auth0.com/docs/get-started/apis"> APIs</a>.

        Parameters
        ----------
        identifier : str
            Unique identifier for the API used as the audience parameter on authorization calls. Can not be changed once set.

        name : typing.Optional[str]
            Friendly name for this resource server. Can not contain `<` or `>` characters.

        scopes : typing.Optional[typing.Sequence[ResourceServerScope]]
            List of permissions (scopes) that this API uses.

        signing_alg : typing.Optional[SigningAlgorithmEnum]

        signing_secret : typing.Optional[str]
            Secret used to sign tokens when using symmetric algorithms (HS256).

        allow_offline_access : typing.Optional[bool]
            Whether refresh tokens can be issued for this API (true) or not (false).

        allow_online_access : typing.Optional[bool]
            Whether Online Refresh Tokens can be issued for this API (true) or not (false).

        allow_online_access_with_ephemeral_sessions : typing.Optional[bool]
            Whether Online Refresh Tokens can be issued even when sessions are configured as ephemeral (true) or not (false).

        token_lifetime : typing.Optional[int]
            Expiration value (in seconds) for access tokens issued for this API from the token endpoint.

        token_dialect : typing.Optional[ResourceServerTokenDialectSchemaEnum]

        skip_consent_for_verifiable_first_party_clients : typing.Optional[bool]
            Whether to skip user consent for applications flagged as first party (true) or not (false).

        enforce_policies : typing.Optional[bool]
            Whether to enforce authorization policies (true) or to ignore them (false).

        token_encryption : typing.Optional[ResourceServerTokenEncryption]

        consent_policy : typing.Optional[ResourceServerConsentPolicyEnum]

        authorization_details : typing.Optional[typing.Sequence[typing.Any]]

        proof_of_possession : typing.Optional[ResourceServerProofOfPossession]

        subject_type_authorization : typing.Optional[ResourceServerSubjectTypeAuthorization]

        authorization_policy : typing.Optional[ResourceServerAuthorizationPolicy]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateResourceServerResponseContent
            Resource server successfully created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.resource_servers.create(
                identifier="identifier",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            identifier=identifier,
            name=name,
            scopes=scopes,
            signing_alg=signing_alg,
            signing_secret=signing_secret,
            allow_offline_access=allow_offline_access,
            allow_online_access=allow_online_access,
            allow_online_access_with_ephemeral_sessions=allow_online_access_with_ephemeral_sessions,
            token_lifetime=token_lifetime,
            token_dialect=token_dialect,
            skip_consent_for_verifiable_first_party_clients=skip_consent_for_verifiable_first_party_clients,
            enforce_policies=enforce_policies,
            token_encryption=token_encryption,
            consent_policy=consent_policy,
            authorization_details=authorization_details,
            proof_of_possession=proof_of_possession,
            subject_type_authorization=subject_type_authorization,
            authorization_policy=authorization_policy,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self,
        id: str,
        *,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetResourceServerResponseContent:
        """
        Retrieve <a href="https://auth0.com/docs/apis">API</a> details with the given ID.

        Parameters
        ----------
        id : str
            ID or audience of the resource server to retrieve.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetResourceServerResponseContent
            Resource server successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.resource_servers.get(
                id="id",
                include_fields=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, include_fields=include_fields, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete an existing API by ID. For more information, read <a href="https://www.auth0.com/docs/get-started/apis/api-settings">API Settings</a>.

        Parameters
        ----------
        id : str
            ID or the audience of the resource server to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.resource_servers.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        scopes: typing.Optional[typing.Sequence[ResourceServerScope]] = OMIT,
        signing_alg: typing.Optional[SigningAlgorithmEnum] = OMIT,
        signing_secret: typing.Optional[str] = OMIT,
        skip_consent_for_verifiable_first_party_clients: typing.Optional[bool] = OMIT,
        allow_offline_access: typing.Optional[bool] = OMIT,
        allow_online_access: typing.Optional[bool] = OMIT,
        allow_online_access_with_ephemeral_sessions: typing.Optional[bool] = OMIT,
        token_lifetime: typing.Optional[int] = OMIT,
        token_dialect: typing.Optional[ResourceServerTokenDialectSchemaEnum] = OMIT,
        enforce_policies: typing.Optional[bool] = OMIT,
        token_encryption: typing.Optional[ResourceServerTokenEncryption] = OMIT,
        consent_policy: typing.Optional[ResourceServerConsentPolicyEnum] = OMIT,
        authorization_details: typing.Optional[typing.Sequence[typing.Any]] = OMIT,
        proof_of_possession: typing.Optional[ResourceServerProofOfPossession] = OMIT,
        subject_type_authorization: typing.Optional[ResourceServerSubjectTypeAuthorization] = OMIT,
        authorization_policy: typing.Optional[ResourceServerAuthorizationPolicy] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateResourceServerResponseContent:
        """
        Change an existing API setting by resource server ID. For more information, read <a href="https://www.auth0.com/docs/get-started/apis/api-settings">API Settings</a>.

        Parameters
        ----------
        id : str
            ID or audience of the resource server to update.

        name : typing.Optional[str]
            Friendly name for this resource server. Can not contain `<` or `>` characters.

        scopes : typing.Optional[typing.Sequence[ResourceServerScope]]
            List of permissions (scopes) that this API uses.

        signing_alg : typing.Optional[SigningAlgorithmEnum]

        signing_secret : typing.Optional[str]
            Secret used to sign tokens when using symmetric algorithms (HS256).

        skip_consent_for_verifiable_first_party_clients : typing.Optional[bool]
            Whether to skip user consent for applications flagged as first party (true) or not (false).

        allow_offline_access : typing.Optional[bool]
            Whether refresh tokens can be issued for this API (true) or not (false).

        allow_online_access : typing.Optional[bool]
            Whether Online Refresh Tokens can be issued for this API (true) or not (false).

        allow_online_access_with_ephemeral_sessions : typing.Optional[bool]
            Whether Online Refresh Tokens can be issued even when sessions are configured as ephemeral (true) or not (false).

        token_lifetime : typing.Optional[int]
            Expiration value (in seconds) for access tokens issued for this API from the token endpoint.

        token_dialect : typing.Optional[ResourceServerTokenDialectSchemaEnum]

        enforce_policies : typing.Optional[bool]
            Whether authorization policies are enforced (true) or not enforced (false).

        token_encryption : typing.Optional[ResourceServerTokenEncryption]

        consent_policy : typing.Optional[ResourceServerConsentPolicyEnum]

        authorization_details : typing.Optional[typing.Sequence[typing.Any]]

        proof_of_possession : typing.Optional[ResourceServerProofOfPossession]

        subject_type_authorization : typing.Optional[ResourceServerSubjectTypeAuthorization]

        authorization_policy : typing.Optional[ResourceServerAuthorizationPolicy]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateResourceServerResponseContent
            Resource server successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.resource_servers.update(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            id,
            name=name,
            scopes=scopes,
            signing_alg=signing_alg,
            signing_secret=signing_secret,
            skip_consent_for_verifiable_first_party_clients=skip_consent_for_verifiable_first_party_clients,
            allow_offline_access=allow_offline_access,
            allow_online_access=allow_online_access,
            allow_online_access_with_ephemeral_sessions=allow_online_access_with_ephemeral_sessions,
            token_lifetime=token_lifetime,
            token_dialect=token_dialect,
            enforce_policies=enforce_policies,
            token_encryption=token_encryption,
            consent_policy=consent_policy,
            authorization_details=authorization_details,
            proof_of_possession=proof_of_possession,
            subject_type_authorization=subject_type_authorization,
            authorization_policy=authorization_policy,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .log_stream_datadog_enum import LogStreamDatadogEnum
from .log_stream_datadog_sink import LogStreamDatadogSink
from .log_stream_filter import LogStreamFilter
from .log_stream_pii_config import LogStreamPiiConfig
from .log_stream_status_enum import LogStreamStatusEnum


class LogStreamDatadogResponseSchema(UniversalBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The id of the log stream
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    log stream name
    """

    status: typing.Optional[LogStreamStatusEnum] = None
    type: typing.Optional[LogStreamDatadogEnum] = None
    is_priority: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="isPriority"),
        pydantic.Field(alias="isPriority", description="True for priority log streams, false for non-priority"),
    ] = None
    """
    True for priority log streams, false for non-priority
    """

    filters: typing.Optional[typing.List[LogStreamFilter]] = pydantic.Field(default=None)
    """
    Only logs events matching these filters will be delivered by the stream. If omitted or empty, all events will be delivered.
    """

    pii_config: typing.Optional[LogStreamPiiConfig] = None
    sink: typing.Optional[LogStreamDatadogSink] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.user_multifactor_provider_enum import UserMultifactorProviderEnum
from .raw_client import AsyncRawMultifactorClient, RawMultifactorClient


class MultifactorClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawMultifactorClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawMultifactorClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawMultifactorClient
        """
        return self._raw_client

    def invalidate_remember_browser(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Invalidate all remembered browsers across all [authentication factors](https://auth0.com/docs/multifactor-authentication) for a user.

        Parameters
        ----------
        id : str
            ID of the user to invalidate all remembered browsers and authentication factors for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.users.multifactor.invalidate_remember_browser(
            id="id",
        )
        """
        _response = self._raw_client.invalidate_remember_browser(id, request_options=request_options)
        return _response.data

    def delete_provider(
        self, id: str, provider: UserMultifactorProviderEnum, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Remove a [multifactor](https://auth0.com/docs/multifactor-authentication) authentication configuration from a user's account. This forces the user to manually reconfigure the multi-factor provider.

        Parameters
        ----------
        id : str
            ID of the user to remove a multifactor configuration from.

        provider : UserMultifactorProviderEnum
            The multi-factor provider. Supported values 'duo' or 'google-authenticator'

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.users.multifactor.delete_provider(
            id="id",
            provider="duo",
        )
        """
        _response = self._raw_client.delete_provider(id, provider, request_options=request_options)
        return _response.data


class AsyncMultifactorClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawMultifactorClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawMultifactorClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawMultifactorClient
        """
        return self._raw_client

    async def invalidate_remember_browser(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Invalidate all remembered browsers across all [authentication factors](https://auth0.com/docs/multifactor-authentication) for a user.

        Parameters
        ----------
        id : str
            ID of the user to invalidate all remembered browsers and authentication factors for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.multifactor.invalidate_remember_browser(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.invalidate_remember_browser(id, request_options=request_options)
        return _response.data

    async def delete_provider(
        self, id: str, provider: UserMultifactorProviderEnum, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Remove a [multifactor](https://auth0.com/docs/multifactor-authentication) authentication configuration from a user's account. This forces the user to manually reconfigure the multi-factor provider.

        Parameters
        ----------
        id : str
            ID of the user to remove a multifactor configuration from.

        provider : UserMultifactorProviderEnum
            The multi-factor provider. Supported values 'duo' or 'google-authenticator'

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.multifactor.delete_provider(
                id="id",
                provider="duo",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_provider(id, provider, request_options=request_options)
        return _response.data



from .conftest import get_client, verify_request_count

from auth0.management import CustomSigningKeyJwk


def test_keys_customSigning_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "keys.custom_signing.get.0"
    client = get_client(test_id)
    client.keys.custom_signing.get()
    verify_request_count(test_id, "GET", "/keys/custom-signing", None, 1)


def test_keys_customSigning_set_() -> None:
    """Test set endpoint with WireMock"""
    test_id = "keys.custom_signing.set_.0"
    client = get_client(test_id)
    client.keys.custom_signing.set(
        keys=[
            CustomSigningKeyJwk(
                kty="EC",
            )
        ],
    )
    verify_request_count(test_id, "PUT", "/keys/custom-signing", None, 1)


def test_keys_customSigning_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "keys.custom_signing.delete.0"
    client = get_client(test_id)
    client.keys.custom_signing.delete()
    verify_request_count(test_id, "DELETE", "/keys/custom-signing", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ConnectionPasswordOptionsHistory(UniversalBaseModel):
    """
    Password history policy configuration to prevent password reuse
    """

    active: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enables password history checking to prevent users from reusing recent passwords
    """

    size: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of previous passwords to remember and prevent reuse (1-24)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .event_stream_action_destination import EventStreamActionDestination
from .event_stream_webhook_destination import EventStreamWebhookDestination

EventStreamDestinationPatch = typing.Union[EventStreamWebhookDestination, EventStreamActionDestination]



# This file was auto-generated by Fern from our API Definition.

import typing

from .certificate_subject_dn_credential import CertificateSubjectDnCredential

ClientCreateAuthenticationMethodTlsClientAuthCredentials = typing.List[CertificateSubjectDnCredential]
"""
Fully defined credentials that will be enabled on the client for CA-based mTLS authentication.
"""



# This file was auto-generated by Fern from our API Definition.

from .connection_scope_array import ConnectionScopeArray

ConnectionOptionsScopeGithub = ConnectionScopeArray
"""
Scope options for GitHub connections.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class IntegrationSemVer(UniversalBaseModel):
    """
    Semver denotes the major.minor version of an integration release
    """

    major: typing.Optional[int] = pydantic.Field(default=None)
    """
    Major is the major number of a semver
    """

    minor: typing.Optional[int] = pydantic.Field(default=None)
    """
    Minior is the minior number of a semver
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionDisplayValuesSupported = typing.List[str]
"""
JSON array containing a list of the JWS signing algorithms (alg values) supported by the Token Endpoint for the signature on the JWT [JWT] used to authenticate the Client at the Token Endpoint for the private_key_jwt and client_secret_jwt authentication methods. Servers SHOULD support RS256. The value none MUST NOT be used.
"""



from .conftest import get_client, verify_request_count


def test_anomaly_blocks_check_ip() -> None:
    """Test checkIp endpoint with WireMock"""
    test_id = "anomaly.blocks.check_ip.0"
    client = get_client(test_id)
    client.anomaly.blocks.check_ip(
        id="id",
    )
    verify_request_count(test_id, "GET", "/anomaly/blocks/ips/id", None, 1)


def test_anomaly_blocks_unblock_ip() -> None:
    """Test unblockIp endpoint with WireMock"""
    test_id = "anomaly.blocks.unblock_ip.0"
    client = get_client(test_id)
    client.anomaly.blocks.unblock_ip(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/anomaly/blocks/ips/id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from .connection_client_id_line import ConnectionClientIdLine
from .connection_client_secret_line import ConnectionClientSecretLine
from .connection_options_common import ConnectionOptionsCommon
from .connection_scope_array import ConnectionScopeArray
from .connection_set_user_root_attributes_enum import ConnectionSetUserRootAttributesEnum
from .connection_upstream_params import ConnectionUpstreamParams


class ConnectionOptionsLine(ConnectionOptionsCommon):
    """
    Options for the 'line' connection
    """

    client_id: typing.Optional[ConnectionClientIdLine] = None
    client_secret: typing.Optional[ConnectionClientSecretLine] = None
    freeform_scopes: typing.Optional[ConnectionScopeArray] = None
    scope: typing.Optional[ConnectionScopeArray] = None
    set_user_root_attributes: typing.Optional[ConnectionSetUserRootAttributesEnum] = None
    upstream_params: typing.Optional[ConnectionUpstreamParams] = None
    email: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Permission to request the user's email address from LINE. When enabled, adds the 'email' scope to OAuth requests. Note: LINE requires special approval to access user email addresses.
    """

    profile: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Permission to request the user's basic profile information from LINE. When enabled, adds the 'profile' scope to OAuth requests. LINE requires this scope to retrieve user display name, profile picture, and status message.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import encode_path_param
from ...core.pagination import AsyncPager, SyncPager
from ...core.parse_error import ParsingError
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...errors.forbidden_error import ForbiddenError
from ...errors.not_found_error import NotFoundError
from ...errors.too_many_requests_error import TooManyRequestsError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.list_user_organizations_offset_paginated_response_content import (
    ListUserOrganizationsOffsetPaginatedResponseContent,
)
from ...types.organization import Organization
from pydantic import ValidationError


class RawOrganizationsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Organization, ListUserOrganizationsOffsetPaginatedResponseContent]:
        """
        Retrieve list of the specified user's current Organization memberships. User must be specified by user ID. For more information, review [Auth0 Organizations](https://auth0.com/docs/manage-users/organizations).

        Parameters
        ----------
        id : str
            ID of the user to retrieve the organizations for.

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Defaults to 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Organization, ListUserOrganizationsOffsetPaginatedResponseContent]
            Organizations successfully retrieved.
        """
        page = page if page is not None else 0

        _response = self._client_wrapper.httpx_client.request(
            f"users/{encode_path_param(id)}/organizations",
            method="GET",
            params={
                "page": page,
                "per_page": per_page,
                "include_totals": include_totals,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListUserOrganizationsOffsetPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListUserOrganizationsOffsetPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.organizations
                _has_next = len(_items or []) > 0
                _get_next = lambda: self.list(
                    id,
                    page=page + 1,
                    per_page=per_page,
                    include_totals=include_totals,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawOrganizationsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Organization, ListUserOrganizationsOffsetPaginatedResponseContent]:
        """
        Retrieve list of the specified user's current Organization memberships. User must be specified by user ID. For more information, review [Auth0 Organizations](https://auth0.com/docs/manage-users/organizations).

        Parameters
        ----------
        id : str
            ID of the user to retrieve the organizations for.

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Defaults to 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Organization, ListUserOrganizationsOffsetPaginatedResponseContent]
            Organizations successfully retrieved.
        """
        page = page if page is not None else 0

        _response = await self._client_wrapper.httpx_client.request(
            f"users/{encode_path_param(id)}/organizations",
            method="GET",
            params={
                "page": page,
                "per_page": per_page,
                "include_totals": include_totals,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListUserOrganizationsOffsetPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListUserOrganizationsOffsetPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.organizations
                _has_next = len(_items or []) > 0

                async def _get_next():
                    return await self.list(
                        id,
                        page=page + 1,
                        per_page=per_page,
                        include_totals=include_totals,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



from .conftest import get_client, verify_request_count


def test_organizations_members_effectiveRoles_sources_groups_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "organizations.members.effective_roles.sources.groups.list_.0"
    client = get_client(test_id)
    client.organizations.members.effective_roles.sources.groups.list(
        id="id",
        user_id="user_id",
        from_="from",
        take=1,
        role_id="role_id",
    )
    verify_request_count(
        test_id,
        "GET",
        "/organizations/id/members/user_id/effective-roles/sources/groups",
        {"from": "from", "take": "1", "role_id": "role_id"},
        1,
    )



# This file was auto-generated by Fern from our API Definition.

import typing

CreateConnectionRequestContentSalesforceSandboxStrategy = typing.Union[typing.Literal["salesforce-sandbox"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .native_social_login_apple_patch import NativeSocialLoginApplePatch
from .native_social_login_facebook_patch import NativeSocialLoginFacebookPatch
from .native_social_login_google_patch import NativeSocialLoginGooglePatch


class NativeSocialLoginPatch(UniversalBaseModel):
    """
    Configure native social settings
    """

    apple: typing.Optional[NativeSocialLoginApplePatch] = None
    facebook: typing.Optional[NativeSocialLoginFacebookPatch] = None
    google: typing.Optional[NativeSocialLoginGooglePatch] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

UserAttributeProfileUserIdOidcStrategyOverrideMapping = typing.Union[typing.Literal["sub", "oid", "email"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from .connection_options_common import ConnectionOptionsCommon
from .connection_set_user_root_attributes_enum import ConnectionSetUserRootAttributesEnum
from .connection_upstream_params import ConnectionUpstreamParams


class ConnectionOptionsApple(ConnectionOptionsCommon):
    """
    Options for the 'apple' connection
    """

    app_secret: typing.Optional[str] = pydantic.Field(default=None)
    """
    Apple App Secret (must be a PEM)
    """

    client_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Apple Services ID
    """

    email: typing.Optional[bool] = pydantic.Field(default=None)
    """
    User has the option to obfuscate the email with Apple's relay service
    """

    freeform_scopes: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Array of freeform scopes
    """

    kid: typing.Optional[str] = pydantic.Field(default=None)
    """
    Apple Key ID
    """

    name: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether to request name from Apple
    """

    scope: typing.Optional[str] = pydantic.Field(default=None)
    """
    Space separated list of scopes
    """

    set_user_root_attributes: typing.Optional[ConnectionSetUserRootAttributesEnum] = None
    team_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Apple Team ID
    """

    upstream_params: typing.Optional[ConnectionUpstreamParams] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .connection_email_body_email import ConnectionEmailBodyEmail
from .connection_email_email_syntax import ConnectionEmailEmailSyntax
from .connection_email_from_email import ConnectionEmailFromEmail
from .connection_email_subject_email import ConnectionEmailSubjectEmail


class ConnectionEmailEmail(UniversalBaseModel):
    """
    Email template configuration
    """

    body: typing.Optional[ConnectionEmailBodyEmail] = None
    from_: typing_extensions.Annotated[
        typing.Optional[ConnectionEmailFromEmail], FieldMetadata(alias="from"), pydantic.Field(alias="from")
    ] = None
    subject: typing.Optional[ConnectionEmailSubjectEmail] = None
    syntax: typing.Optional[ConnectionEmailEmailSyntax] = pydantic.Field(default=None)
    """
    Email template syntax type
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
